@@ -22,6 +22,7 @@ from datetime import datetime
 ReplData = namedtuple('ReplData', ['value', 'font', 'suffix'])
 KeyRepl  = namedtuple('KeyRepl',  ['bk_stop', 'silent', 'spaced', 'repl'])
 KeyData  = namedtuple('KeyData',  ['stamp', 'is_ctrl', 'bk_stop', 'silent', 'spaced', 'markup'])
+ButtonData = namedtuple('ButtonData',  ['stamp', 'btn', 'pressed'])
 
 REPLACE_SYMS = {
     # Regular keys
@@ -165,14 +166,16 @@ def keysym_to_mod(keysym):
 
 
 class LabelManager(object):
-    def __init__(self, listener, logger, key_mode, bak_mode, mods_mode, mods_only,
-                 multiline, vis_shift, vis_space, recent_thr, compr_cnt, ignore,
-                 pango_ctx, enabled):
+    def __init__(self, label_listener, image_listener, logger, key_mode,
+                 bak_mode, mods_mode, mods_only, multiline, vis_shift,
+                 vis_space, recent_thr, compr_cnt, ignore, pango_ctx,
+                 enabled):
         self.key_mode = key_mode
         self.bak_mode = bak_mode
         self.mods_mode = mods_mode
         self.logger = logger
-        self.listener = listener
+        self.label_listener = label_listener
+        self.image_listener = image_listener
         self.data = []
         self.enabled = enabled
         self.mods_only = mods_only
@@ -195,7 +198,9 @@ class LabelManager(object):
         self.stop()
         compose = (self.key_mode == 'composed')
         translate = (self.key_mode in ['composed', 'translated'])
-        self.kl = InputListener(self.key_press, InputType.keyboard, compose, translate)
+        self.kl = InputListener(self.key_press, self.btn_press,
+                                InputType.keyboard | InputType.button, compose,
+                                translate)
         self.kl.start()
         self.logger.debug("Thread started.")
 
@@ -304,7 +309,7 @@ class LabelManager(object):
         if recent:
             markup += '</u>'
         self.logger.debug("Label updated: %s." % repr(markup))
-        self.listener(markup, synthetic)
+        self.label_listener(markup, synthetic)
 
 
     def queue_update(self):
@@ -314,9 +319,22 @@ class LabelManager(object):
     def key_press(self, event):
         if event is None:
             self.logger.debug("inputlistener failure: {}".format(str(self.kl.error)))
-            self.listener(None, None)
+            self.label_listener(None, None)
+            return
+        if event.symbol is None:
+            # TODO: Investigate what causes this to happen.
+            # I caught it once in pdb, but in this function, not in inputlistener,
+            # and KeyData doesn't contain enough info.
             return
         symbol = event.symbol.decode()
+
+        if self.enabled:
+            for mod, button_id in zip(['ctrl', 'alt', 'shift'], range(8, 11)):
+                if symbol in MODS_SYMS[mod]:
+                    self.image_listener(ButtonData(
+                        datetime.now(), button_id, event.pressed
+                    ))
+
         if event.pressed == False:
             self.logger.debug("Key released {:5}(ks): {}".format(event.keysym, symbol))
             return
@@ -337,6 +355,8 @@ class LabelManager(object):
                and symbol in MODS_SYMS[mod]:
                 self.enabled = not self.enabled
                 state = 'enabled' if self.enabled else 'disabled'
+                if not self.enabled:
+                    self.image_listener(None)
                 self.logger.info("{mod}+{mod} detected: screenkey {state}".format(
                     mod=mod.capitalize(), state=state))
         if not self.enabled:
@@ -493,3 +513,18 @@ class LabelManager(object):
             value = event.string or symbol
         self.data.append(KeyData(datetime.now(), True, True, True, True, value))
         return True
+
+
+    def btn_press(self, event):
+        if not self.enabled:
+            return False
+
+        if event.pressed:
+            action = "pressed"
+        else:
+            action = "released"
+        self.logger.debug("Mouse button %d %s" % (event.btn, action))
+
+        self.image_listener(
+            ButtonData(datetime.now(), event.btn, event.pressed)
+        )
